Documentation
¶
Index ¶
Constants ¶
const ( NoHealth = 1 << 0 NoItems = 1 << 1 WeaponsStay = 1 << 2 NoFalling = 1 << 3 InstantItems = 1 << 4 SameLevel = 1 << 5 SkinTeams = 1 << 6 ModelTeams = 1 << 7 NoFriendlyFire = 1 << 8 SpawnFarthest = 1 << 9 ForceRespawn = 1 << 10 NoArmor = 1 << 11 AllowExit = 1 << 12 InfiniteAmmo = 1 << 13 QuadDrop = 1 << 14 FixedFOV = 1 << 15 )
Variables ¶
var ( FreeForAllServer = WeaponsStay | InstantItems | ForceRespawn TDMServer = SpawnFarthest | ForceRespawn | QuadDrop )
Functions ¶
func Modify ¶
Modify will return a new dmflag value based on a string of instructions. Each instruction should be prefixed with either "+" or "-" for including or excluding and separated by spaces. Each is an abbreviation of the related flag, e.g. "ia" for "infinite ammo", "qd" for "quad drop", etc.
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.